/*
 * (C) 2007-2012 Alibaba Group Holding Limited.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.taobao.gecko.service;

import java.nio.channels.FileChannel;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import com.taobao.gecko.core.buffer.IoBuffer;
import com.taobao.gecko.core.command.RequestCommand;
import com.taobao.gecko.core.command.ResponseCommand;
import com.taobao.gecko.core.nio.impl.TimerRef;
import com.taobao.gecko.service.exception.NotifyRemotingException;


/**
 * 
 * Notify Remotingӿ
 * 
 * @author boyan
 * 
 * @since 1.0, 2009-12-16 02:15:02
 */

public interface RemotingController {

    /**
     * ѡĬΪѡ
     * 
     * @param selector
     */
    public void setConnectionSelector(ConnectionSelector selector);


    /**
     * Remoting
     * 
     * @throws NotifyRemotingException
     * 
     */
    public void start() throws NotifyRemotingException;


    /**
     * رRemoting
     * 
     * @throws NotifyRemotingException
     * 
     */
    public void stop() throws NotifyRemotingException;


    /**
     * жͨѶǷ
     * 
     * @return
     */
    public boolean isStarted();


    /**
     * ע
     * 
     * @param <T>
     * @param commandClazz
     * @param processor
     */
    public <T extends RequestCommand> void registerProcessor(Class<T> commandClazz, RequestProcessor<T> processor);


    /**
     * ȡcommandӦĴ
     * 
     * @param clazz
     * @return
     */
    public RequestProcessor<? extends RequestCommand> getProcessor(Class<? extends RequestCommand> clazz);


    /**
     * ȡע,رȡĴ
     * 
     * @param clazz
     * @return
     */
    public RequestProcessor<? extends RequestCommand> unreigsterProcessor(Class<? extends RequestCommand> clazz);


    /**
     * 
     * 
     * @param <T>
     * @param map
     */
    public void addAllProcessors(Map<Class<? extends RequestCommand>, RequestProcessor<? extends RequestCommand>> map);


    /**
     * һʱ
     * 
     * @param timeout
     *            ʱʱ
     * @param timeUnit
     *            ʱ䵥λ
     * @param runnable
     *            ʱִе
     */
    public void insertTimer(TimerRef timerRef);


    /**
     * 첽Ϣ飬ÿݲѡһӷָͣصͳʱʱ䣬ʱһʱӦص
     * 
     * @param groupObjects
     *            group->message
     * @param listener
     *            Ӧ
     * @param timeout
     *            ʱʱ
     * @param timeUnit
     *            ʱ䵥λ
     * @param args
     *            
     */
    public void sendToGroups(Map<String, RequestCommand> groupObjects, MultiGroupCallBackListener listener,
            long timeout, TimeUnit timeUnit, Object... args) throws NotifyRemotingException;


    /**
     * 첽Ϣ
     * 
     * @param groupObjects
     */
    public void sendToGroups(Map<String, RequestCommand> groupObjects) throws NotifyRemotingException;


    /**
     * 첽͸
     * 
     * @param command
     */
    public void sendToAllConnections(RequestCommand command) throws NotifyRemotingException;


    /**
     * 첽͸ָеһӣĬ
     * 
     * @param group
     * @param command
     */
    public void sendToGroup(String group, RequestCommand command) throws NotifyRemotingException;


    /**
     * ָFileChannelpositionλÿʼsizeֽڵָgroupһsocket,
     * remotingḺָ֤Сݴsocketfile channelݲsizeСʵʴС䡣
     * headtailָڴļ֮ǰ֮ҪдݣΪnullǺļΪһ͡
     * ָĳʱʱȡ(ûпʼĻ,Ѿʼ޷ֹ)֪ͨlistener
     * 
     * @param group
     * @param head
     * @param tail
     * @param channel
     * @param position
     * @param size
     * @param opaque
     * @param listener
     * @param time
     * @param unit
     * @throws NotifyRemotingException
     */
    public void transferToGroup(String group, IoBuffer head, IoBuffer tail, FileChannel channel, long position,
            long size, Integer opaque, SingleRequestCallBackListener listener, long time, TimeUnit unit)
            throws NotifyRemotingException;


    /**
     * ݵָgroupĳsocketӣҪʹõʱδ֪Ҳȡ
     * 
     * @see #transferToGroup(String, IoBuffer, IoBuffer, FileChannel, long,
     *      long, Integer, SingleRequestCallBackListener, long, TimeUnit)
     * @param group
     * @param head
     * @param tail
     * @param channel
     * @param position
     * @param size
     */
    public void transferToGroup(String group, IoBuffer head, IoBuffer tail, FileChannel channel, long position,
            long size) throws NotifyRemotingException;


    /**
     * 첽͸ָ
     * 
     * @param group
     * @param command
     */
    public void sendToGroupAllConnections(String group, RequestCommand command) throws NotifyRemotingException;


    /**
     * 첽͸ָеһӣָصRequestCallBackListenerĬϲĬϳʱΪ1,
     * ʱʱ佫һʱӦص
     * 
     * @param group
     *            
     * @param command
     *            
     * @param listener
     *            Ӧ
     */
    public void sendToGroup(String group, RequestCommand command, SingleRequestCallBackListener listener)
            throws NotifyRemotingException;


    /**
     * 첽͸ָеһӣĬϲָʱ,ʱʱ佫һʱӦص
     * 
     * @param group
     *            
     * @param command
     *            
     * @param listener
     *            Ӧ
     */
    public void sendToGroup(String group, RequestCommand command, SingleRequestCallBackListener listener, long time,
            TimeUnit timeunut) throws NotifyRemotingException;


    /**
     * ͬ÷еһӣĬϳʱ1
     * 
     * @param group
     *            
     * @param command
     *            
     * @return
     * @throws InterruptedException
     * @throws TimeoutException
     */
    public ResponseCommand invokeToGroup(String group, RequestCommand command) throws InterruptedException,
            TimeoutException, NotifyRemotingException;


    /**
     * ͬ÷еһӣָʱʱ
     * 
     * @param group
     *            
     * @param command
     *            
     * @param time
     *            ʱʱ
     * @param timeUnit
     *            ʱ䵥λ
     * @return
     * @throws InterruptedException
     * @throws TimeoutException
     */
    public ResponseCommand invokeToGroup(String group, RequestCommand command, long time, TimeUnit timeUnit)
            throws InterruptedException, TimeoutException, NotifyRemotingException;


    /**
     * 첽͸ָӣĬϳʱ1,ʱʱ佫һʱӦص
     * 
     * @param group
     *            
     * @param command
     *            
     * @param listener
     *            Ӧ
     */
    public void sendToGroupAllConnections(String group, RequestCommand command,
            GroupAllConnectionCallBackListener listener) throws NotifyRemotingException;


    /**
     * ͬ÷ڵӣ
     * ʱӦӽһBooleanResponseCommandΪresponseStatusΪTIMEOUT
     * ,ûӽnull
     * 
     * @param group
     * @param command
     * @return
     * @throws InterruptedException
     * @throws NotifyRemotingException
     */
    public Map<Connection, ResponseCommand> invokeToGroupAllConnections(String group, RequestCommand command)
            throws InterruptedException, NotifyRemotingException;


    /**
     * ͬ÷ڵӣ
     * ʱӦӽһBooleanResponseCommandΪresponseStatusΪTIMEOUT
     * ,ûӽnull
     * 
     * @param group
     * @param command
     * @return
     * @throws InterruptedException
     * @throws NotifyRemotingException
     */
    public Map<Connection, ResponseCommand> invokeToGroupAllConnections(String group, RequestCommand command,
            long time, TimeUnit timeUnit) throws InterruptedException, NotifyRemotingException;


    /**
     * 첽͸ָӣָʱʱ䣬ʱʱ佫һʱӦص
     * 
     * @param group
     *            
     * @param command
     *            
     * @param listener
     *            Ӧ
     */
    public void sendToGroupAllConnections(String group, RequestCommand command,
            GroupAllConnectionCallBackListener listener, long time, TimeUnit timeUnit) throws NotifyRemotingException;


    /**
     * ȡgroupӦ
     * 
     * @param group
     * @return
     */
    public int getConnectionCount(String group);


    /**
     * ȡgroup
     * 
     * @return
     */
    public Set<String> getGroupSet();


    /**
     * 
     * 
     * @param group
     * 
     * @param key
     * @param value
     */
    public void setAttribute(String group, String key, Object value);


    /**
     * ԣConcurrentHashMap.putIfAbsent
     * 
     * @param group
     * 
     * @param key
     * @param value
     * @return
     */
    public Object setAttributeIfAbsent(String group, String key, Object value);


    /**
     * ȡ
     * 
     * @param group
     * 
     * @param key
     * @return
     */
    public Object getAttribute(String group, String key);


    /**
     * ڼ
     * 
     * @param connectionLifeCycleListener
     */
    public void addConnectionLifeCycleListener(ConnectionLifeCycleListener connectionLifeCycleListener);


    /**
     * ڼ
     * 
     * @param connectionLifeCycleListener
     */
    public void removeConnectionLifeCycleListener(ConnectionLifeCycleListener connectionLifeCycleListener);


    /**
     * Ƴ
     * 
     * @param group
     * 
     * @param key
     * @return
     */
    public Object removeAttribute(String group, String key);


    /**
     * ȡȫ
     * 
     * @return
     */
    public RemotingContext getRemotingContext();


    /**
     * ݲԴӷеѡһ
     * 
     * @param group
     * @param connectionSelector
     *            ѡ
     * @param request
     *            ͵
     * @return
     */
    public Connection selectConnectionForGroup(String group, ConnectionSelector connectionSelector,
            RequestCommand request) throws NotifyRemotingException;

}